import { TABLE_NAME } from 'src/core/strings/constants/table.constants';
import { Column, Entity, JoinColumn, ManyToOne, OneToMany } from 'typeorm';
import { BaseCoreModel } from 'src/core/modules/data/model/base-core.model';
import { TransactionItemEntity } from '../../domain/entities/transaction-item.entity';
import { TransactionModel } from './transaction.model';
import { RefundItemModel } from 'src/modules/transaction/refund/data/models/refund-item.model';

@Entity(TABLE_NAME.TRANSACTION_ITEM)
export class TransactionItemModel
  extends BaseCoreModel<TransactionItemEntity>
  implements TransactionItemEntity
{
  // item data
  @Column('varchar', { name: 'item_id', nullable: true })
  item_id: string;

  @Column('varchar', { name: 'qr_image_url', nullable: true })
  qr_image_url: string;

  @Column('varchar', { name: 'item_name', nullable: true })
  item_name: string;

  @Column('varchar', { name: 'item_type', nullable: true })
  item_type: string;

  @Column('bigint', { name: 'item_price', nullable: true })
  item_price: number;

  @Column('bigint', { name: 'item_hpp', nullable: true })
  item_hpp: number;

  @Column('varchar', { name: 'item_category_id', nullable: true })
  item_category_id: string;

  @Column('varchar', { name: 'item_category_name', nullable: true })
  item_category_name: string;

  @Column('json', { name: 'item_bundlings', nullable: true })
  item_bundlings: string;

  // item tenant data
  @Column('varchar', { name: 'item_tenant_id', nullable: true })
  item_tenant_id: string;

  @Column('varchar', { name: 'item_tenant_name', nullable: true })
  item_tenant_name: string;

  @Column('decimal', { name: 'item_tenant_share_margin', nullable: true })
  item_tenant_share_margin: number;

  // calculation data
  @Column('decimal', { name: 'total_price', nullable: true })
  total_price: number;

  @Column('decimal', { name: 'total_hpp', nullable: true })
  total_hpp: number;

  @Column('decimal', { name: 'total_profit', nullable: true })
  total_profit: number;

  @Column('decimal', { name: 'total_share_tenant', nullable: true })
  total_share_tenant: number;

  @Column('int', { name: 'qty', nullable: true })
  qty: number;

  @Column('int', { name: 'qty_remaining', nullable: true })
  qty_remaining: number;

  @Column('json', { name: 'taxes', nullable: true })
  taxes: string;

  @Column('varchar', { name: 'transaction_id', nullable: true })
  transaction_id: string;
  @ManyToOne(() => TransactionModel, (model) => model.items, {
    onDelete: 'CASCADE',
    onUpdate: 'CASCADE',
  })
  @JoinColumn({ name: 'transaction_id' })
  transaction: TransactionModel;

  // relations to refund
  @OneToMany(() => RefundItemModel, (model) => model.transaction_item, {
    cascade: true,
    onDelete: 'CASCADE',
    onUpdate: 'CASCADE',
  })
  refunds: RefundItemModel[];
}
